"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = createRegexTransformStream;
exports.replaceStream = replaceStream;

var _stream = require("stream");

var _lib = require("./lib");

/**
 * Create a Transform stream which will maintain a buffer with data received from a Readable stream and write data when the buffer can be matched against the regex. It will push the whole match object (or objects when the g flag is used) returned by `/regex/.exec(buffer)`.
 * @param {RegExp} regex Regular expression to execute
 */
function createRegexTransformStream(regex) {
  let buffer = '';
  const ts = new _stream.Transform({
    transform(chunk, encoding, next) {
      let lastMatch;
      let match;
      buffer += chunk.toString(); // thx stream-snitch
      // https://github.com/dmotz/stream-snitch/blob/master/index.js#L52
      // eslint-disable-next-line no-cond-assign

      while (match = regex.exec(buffer)) {
        ts.push(match);
        lastMatch = match;
        if (!regex.global) break;
      }

      if (lastMatch) {
        buffer = buffer.slice(lastMatch.index + lastMatch[0].length);
      }

      next();
    },

    objectMode: true
  });
  return ts;
}
/**
 * Create a replacement stream, which will push data when it's done replacing each incoming chunk. The return is a transform stream which writes strings. If the replacement is passed as a function, it will work in the same way as the replacer for `string.replace` method (https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Global_Objects/String/replace), taking the `match` as the first argument, and matched `p1`, `p2`, _etc_ parameters as following arguments.
 * @param {RegexObject|RegexObject[]} rules A single replacement rule, or multiple rules.
 * @example
 *
 * // markdown __ to html emphasise implementation
 * const stream = replaceStream({
 *  re: /__(\S+)__/g,
 *  replacement(match, p1) {
 *    return `<em>${p1}</em>`
 *  },
 * })
 */


function replaceStream(rules) {
  const re = Array.isArray(rules) ? rules : [rules];
  const fr = re.filter(_lib.checkRegexObject);
  const ts = new _stream.Transform({
    transform(chunk, _, next) {
      let string = `${chunk}`;
      fr.forEach(({
        re,
        replacement
      }) => {
        string = string.replace(re, replacement);
      });
      ts.push(string);
      next();
    }

  });
  return ts;
}
/**
 * @typedef {(substring: string, ...args: any[]) => string} Replacer
 *
 * @typedef {Object} RegexObject
 * @property {RegExp} re Regular expression to match against
 * @property {string|Replacer} replacement A replacement string, or a replacement string function.
 */